
RELEASE_VERSION='0.3.1422958728'
BUILD_DATE='Thu Aug 22 09:18:07 UTC 2024'
BUILD_MACHINE_INFO='Linux runner-jhcjxvh-project-28359933-concurrent-0 5.15.154+ #1 SMP Sat May 4 12:14:42 UTC 2024 x86_64 GNU/Linux'
GITLAB_CI_JOB_ID='7640412494'
GIT_COMMIT_SHA='7c2bd451320b905d7366196067fcfb13a719b4de'
GIT_BRANCH='main'
